﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CbsDownloader
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private string[] urls;

        private System.Windows.Forms.Timer uiTimer;
        private Thread worker;

        private int fileDownloadProgress;
        private int fileDownloadTotal;
        private int totalDownloadProgress;
        private int totalDownloadTotal;
        private string downloadDest;

        private void btnStart_Click(object sender, EventArgs e)
        {
            //Load URL cache
            if(urls == null)
            {
                if(File.Exists("urlcache.txt"))
                {
                    urls = File.ReadAllLines("urlcache.txt");
                } else
                {
                    var dialog = new IndexDownloader();
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                        urls = dialog.found;
                        File.WriteAllLines("urlcache.txt", dialog.found);
                    }
                }
            }

            //Search URLs for candidates
            List<string> downloads = new List<string>();
            foreach(var url in urls)
            {
                //Make sure this is a video we care about
                if (!url.StartsWith("http://vodtv-cbslocal.akamaized.net/"))
                    continue;

                //Parse URL parts
                string path = url.Substring("http://vodtv-cbslocal.akamaized.net/".Length);
                string region = path.Split('/')[0];
                string nameHint = GetFileName(url);

                //Parse date
                string[] dateParts = path.Split('/');
                int day = int.Parse(dateParts[4]);
                int month = int.Parse(dateParts[3]);
                int year = int.Parse(dateParts[2]) + 2000;
                DateTime date = new DateTime(year, month, day);

                //Check
                if (date > startDate.Value && date < endDate.Value && region == this.region.Text)
                    downloads.Add(url);
            }

            //Check
            if(downloads.Count == 0)
            {
                MessageBox.Show("No files found to download.");
                return;
            }

            //Prompt
            var fd = new FolderBrowserDialog();
            if (fd.ShowDialog() != DialogResult.OK)
                return;
            downloadDest = fd.SelectedPath + Path.DirectorySeparatorChar;

            //Clear status
            fileDownloadTotal = 0;
            fileDownloadProgress = 0;
            totalDownloadTotal = downloads.Count;
            totalDownloadProgress = 0;
            btnStart.Enabled = false;
            fileCount.Text = downloads.Count + " files";

            //Start timer
            uiTimer = new System.Windows.Forms.Timer();
            uiTimer.Tick += UiTimer_Tick;
            uiTimer.Interval = 50;
            uiTimer.Start();

            //Start worker thread
            worker = new Thread(WorkerThread);
            worker.IsBackground = true;
            worker.Start(downloads);
        }

        private string GetFileName(string url)
        {
            string filename = url.Split('/')[url.Split('/').Length - 1];
            string[] nameHintParts = filename.Split('_');
            return filename.Substring(nameHintParts[0].Length + nameHintParts[1].Length + nameHintParts[2].Length + nameHintParts[3].Length + 4);
        }

        private void UiTimer_Tick(object sender, EventArgs e)
        {
            UpdateBar(progressBarFile, fileDownloadProgress, fileDownloadTotal);
            UpdateBar(progressBarTotal, totalDownloadProgress, totalDownloadTotal);
        }

        private void UpdateBar(ProgressBar bar, long value, long max)
        {
            if (max == 0)
                bar.Value = 0;
            else
                bar.Value = (int)Math.Min(100, ((float)value / max) * 100);
        }

        private void WorkerThread(object ctx)
        {
            //Get list of downloads
            List<string> downloads = (List<string>)ctx;

            //Allocate
            byte[] buffer = new byte[65536];

            //Loop for each
            foreach (var d in downloads)
            {
                //Open HTTP stream
                var request = (HttpWebRequest)WebRequest.Create(d);
                request.Method = "GET";
                var response = (HttpWebResponse)request.GetResponse();
                using (Stream httpStream = response.GetResponseStream())
                {
                    //Set status
                    fileDownloadProgress = 0;
                    fileDownloadTotal = (int)response.ContentLength;
                    Invoke((MethodInvoker)delegate
                    {
                        downloadInfo.Text = (response.ContentLength / 1000 / 1000).ToString() + " MB";
                    });

                    //Get path
                    string path = downloadDest + GetFileName(d);
                    if(!File.Exists(path))
                    {
                        //Begin downloading
                        using (FileStream output = new FileStream(path + ".tmp", FileMode.Create))
                        {
                            int read;
                            do
                            {
                                read = httpStream.Read(buffer, 0, buffer.Length);
                                fileDownloadProgress += read;
                                output.Write(buffer, 0, read);
                            } while (read != 0);
                        }

                        //Finalize
                        File.Move(path + ".tmp", path);
                    }
                }

                //Update
                totalDownloadProgress++;
            }

            //Done
            Invoke((MethodInvoker)delegate
            {
                MessageBox.Show(totalDownloadTotal + " files downloaded!");
                fileCount.Text = "";
                downloadInfo.Text = "";
                btnStart.Enabled = true;
            });
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }
    }
}
