﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;

namespace CbsDownloader
{
    public partial class IndexDownloader : Form
    {
        public IndexDownloader()
        {
            InitializeComponent();
        }

        private const int DOWNLOAD_SIZE = 301539573;
        private const int DECOMPRESSED_SIZE = 1746093479;
        private const int ENTRY_COUNT = 1143894;

        private System.Windows.Forms.Timer uiTimer;
        private Thread worker;

        private StatsStream downloadStream;
        private StatsStream decompressStream;
        private int read;

        private List<string> urls = new List<string>();

        public string[] found;

        private void IndexDownloader_Load(object sender, EventArgs e)
        {
            //Start timer
            uiTimer = new System.Windows.Forms.Timer();
            uiTimer.Tick += UiTimer_Tick;
            uiTimer.Interval = 50;
            uiTimer.Start();

            //Start worker thread
            worker = new Thread(WorkerThread);
            worker.IsBackground = true;
            worker.Start();
        }

        private void UiTimer_Tick(object sender, EventArgs e)
        {
            if (downloadStream != null)
                UpdateBar(progressBarDownload, downloadStream.BytesRead, DOWNLOAD_SIZE);
            if (decompressStream != null)
                UpdateBar(progressBarDecompressing, decompressStream.BytesRead, DECOMPRESSED_SIZE);
            UpdateBar(progressBarReading, read, ENTRY_COUNT);
        }

        private void WorkerThread()
        {
            //Open HTTP stream
            var request = (HttpWebRequest)WebRequest.Create("https://romanport.com/p/cbsindex/output.bin.gz");
            request.Method = "GET";
            var response = (HttpWebResponse)request.GetResponse();
            downloadStream = new StatsStream(response.GetResponseStream());

            //Open GZIP stream
            decompressStream = new StatsStream(new GZipStream(downloadStream, CompressionMode.Decompress, true));

            //Begin reading
            byte[] buffer = new byte[65536];
            int read;
            do
            {
                //Read header
                read = decompressStream.Read(buffer, 0, 12);
                if (read != 12)
                    break;

                //Get lengths
                short headerLen = BitConverter.ToInt16(buffer, 8);
                short contentLen = BitConverter.ToInt16(buffer, 10);

                //Read headers, skip
                if (decompressStream.Read(buffer, 0, headerLen) != headerLen)
                    break;

                //Read content
                if (decompressStream.Read(buffer, 0, contentLen) != contentLen)
                    break;

                //Throw all optimizations out the window and parse XML
                Uri url = ParseXML(buffer, 0, contentLen);

                //Add
                urls.Add(url.AbsoluteUri);
                this.read++;
            } while (read != 0);

            //Done
            Invoke((MethodInvoker)delegate
            {
                //UI Work goes here
                found = urls.ToArray();
                DialogResult = DialogResult.OK;
                Close();
            });
        }

        private static Uri ParseXML(byte[] buffer, int offset, int length)
        {
            string domain = null;
            string path = null;
            Reader xmlReader = new Reader();
            xmlReader.Configure(buffer, offset, length);
            using (XmlReader reader = XmlReader.Create(xmlReader))
            {
                while (reader.ReadToFollowing("meta"))
                {
                    if (reader.GetAttribute("name") == "httpBase")
                    {
                        domain = reader.GetAttribute("content");
                        break;
                    }
                }
                if (reader.ReadToFollowing("video"))
                    path = reader.GetAttribute("src");
            }

            //Validate
            if (domain == null || path == null)
                throw new Exception("Could not read required values from XML!");

            return new Uri(new Uri(domain), "/" + path);
        }

        private void UpdateBar(ProgressBar bar, long value, long max)
        {
            bar.Value = (int)Math.Min(100, ((float)value / max) * 100);
        }

        class Reader : TextReader
        {
            private byte[] buffer;
            private int offset;
            private int length;

            public void Configure(byte[] buffer, int offset, int length)
            {
                this.buffer = buffer;
                this.offset = offset;
                this.length = length;
            }

            public override int Read(char[] buffer, int index, int count)
            {
                int read = 0;
                for (int i = 0; i < count && length > 0; i++)
                {
                    buffer[index + i] = (char)this.buffer[offset++];
                    length--;
                    read++;
                }
                return read;
            }
        }
    }
}
